@@ -81,9 +81,13 @@ setMethod("dist_pdf",
           definition = function(m, q=NULL, log=FALSE) {
             xmin = m$getXmin(); pars = m$getPars()
             if(is.null(q)) q = m$dat
-            q = q[q >= m$xmin]
-            pdf = dpois(q, pars, log=TRUE) - ppois(xmin, pars, lower.tail=FALSE, log.p=TRUE)
-            if(!log) pdf = exp(pdf)
+            pdf = dpois(q, pars, log=TRUE) - ppois(xmin-0.5, pars, lower.tail=FALSE, log.p=TRUE)
+            if(!log) {
+              pdf = exp(pdf)
+              pdf[q < xmin]  = 0
+            } else {
+              pdf[q < xmin] = -Inf
+            }
             pdf
           }
 )
@@ -97,19 +101,18 @@ setMethod("dist_cdf",
           definition = function(m, q=NULL, lower_tail=TRUE) {
             xmin = m$getXmin(); pars = m$getPars()
             if(is.null(pars)) stop("Model parameters not set.")  
-            
-            if(is.null(q)) {
-              q = m$dat
-              q = q[q>=xmin]
-            } 
+            if(is.null(q)) q = m$dat
+
             p = ppois(q, pars, lower.tail=lower_tail) 
             if(lower_tail){
-              C = ppois(xmin-1, pars, lower.tail=FALSE) 
-              (p/C-1/C+1)
+              C = ppois(xmin-0.5, pars, lower.tail=FALSE) 
+              cdf = (p/C-1/C+1)
             } else {
               C = 1-ppois(xmin, pars)
-              p/C
+              cdf = p/C
             }
+            cdf[q < xmin] = 0
+            cdf
           }
 )
 
